#include "font.h"
#include "ui_font.h"
#include <QProcess>
#include <QFileDialog>
#include <KAuth>
#include <QDebug>
#include "../baseui/common.h"

#define FONT_SCALE_FILE  "/etc/profile.d/hidpi.sh"

Font::Font(QWidget *parent) :
    Page(parent),
    ui(new Ui::Font)
{
    ui->setupUi(this);
    QFile f(":/qss/font.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }

    mQtSettings = new Nde::Settings("nde");
    initControls();
    addItem_to_scalingcomboBox();
    connect(ui->fontSizeSpinBox, QOverload<int>::of(&QSpinBox::valueChanged), this, &Font::updateQtFont);
    connect(ui->StandardFontComboBox, QOverload<int>::of(&QComboBox::activated), this, &Font::updateQtFont);
    connect(ui->MonospaceFontComboBox, QOverload<int>::of(&QComboBox::activated), this, &Font::updateQtFont);
    connect(ui->installFontFrame,SIGNAL(clicked()),this,SLOT(on_installBtn_clicked()));
    connect(ui->ScalingcomboBox,SIGNAL(currentIndexChanged(int)),this,SLOT(on_scaling_changed()));
}

Font::~Font()
{
    delete ui;
    delete mQtSettings;
}

void Font::refreshPage()
{
    //
}

void Font::initControls()
{
    // read Qt style settings from Qt Trolltech.conf config
    mQtSettings->beginGroup(QLatin1String("Qt"));

    QString fontName = mQtSettings->value("font").toString();
    QFont font;
    font.fromString(fontName);
    ui->StandardFontComboBox->setCurrentFont(font);

    ui->fontSizeSpinBox->blockSignals(true);
    ui->fontSizeSpinBox->setValue(font.pointSize());
    ui->fontSizeSpinBox->blockSignals(false);

    int fontStyle = 0;
    if(font.bold())
      fontStyle = font.italic() ? 3 : 1;
    else if(font.italic())
      fontStyle = 2;
    ui->MonospaceFontComboBox->setCurrentIndex(fontStyle);

    mQtSettings->endGroup();
    update();
}

void Font::updateQtFont()
{
    // FIXME: the change does not apply to some currently running Qt programs.
    // FIXME: does not work with KDE apps
    // TODO: also write the config values to GTK+ config files (gtk-2.0.rc and gtk3/settings.ini)
    // FIXME: the selected font does not apply to our own application. Why?

    QFont font = ui->StandardFontComboBox->currentFont();
    int size = ui->fontSizeSpinBox->value();
    bool bold = false;
    bool italic = false;
    switch(ui->MonospaceFontComboBox->currentIndex())
    {
        case 1:
            bold = true;
            break;
        case 2:
            italic = true;
            break;
        case 3:
          bold = italic = true;
    }

    font.setPointSize(size);
    font.setBold(bold);
    font.setItalic(italic);

    const QString fontStr = font.toString();
    if(mQtSettings->value(QLatin1String("Qt/font")).toString() != fontStr) {
        mQtSettings->beginGroup(QLatin1String("Qt"));
        mQtSettings->setValue("font", fontStr);
        mQtSettings->endGroup();
        mQtSettings->sync();

#ifdef Q_WS_X11
        qt_x11_apply_settings_in_all_apps();
#endif

        update();
    }
}

bool Font::eventFilter(QObject *watched, QEvent *event) {
    switch (event->type())
    {
    case QEvent::MouseButtonRelease:
        if(watched == ui->installFontFrame)
        {
            on_installBtn_clicked();
        }
        break;
    default:
        break;
    }
    return  QWidget::eventFilter(watched, event);
}

void Font::on_installBtn_clicked()
{
    QString filePath = QFileDialog::getOpenFileName(this, tr("Install Font"),
                                   "image.img.bz2","File(*.ttf)");


    if (filePath.isNull()) {
        return;
    }

    QFileInfo file(filePath);
    QString fileName = file.fileName();
    QFileInfo dirFile(QString("/usr/share/fonts/") + fileName);
    if(dirFile.exists())
    {
        qDebug() << "Font exist";
        return;
    }
    KAuth::Action action("org.nde.auth.helper.installfont");
    if (!action.isValid()) {
        return ;
    }
    action.setHelperId(QStringLiteral("org.nde.auth.helper"));
    action.addArgument(QStringLiteral("sourceFont"), filePath);
    KAuth::ExecuteJob *job = action.execute();
    if(job->exec())
    {
        int fontId = QFontDatabase::addApplicationFont(filePath);
        QString fontName = QFontDatabase::applicationFontFamilies(fontId).at(0);
        ui->StandardFontComboBox->addItem(fontName);
    }

}

void Font::addItem_to_scalingcomboBox()
{
    QStringList timeList;
    timeList  <<"1" <<"1.25"<<"1.5" <<"1.75"<<"2" ;
    ui->ScalingcomboBox->clear();
    ui->ScalingcomboBox->addItems(timeList);
    getCurrentScaling();
}

 void Font::getCurrentScaling()
 {
     QFile file(FONT_SCALE_FILE);
      if(file.open(QIODevice::ReadOnly|QIODevice::Text))
      {
          QString content = file.readAll();
          QStringList contentList = content.split("\n");
          for(QString line : contentList)
          {
              if(line.contains("FONT_SCALE_LEVEL="))
              {
                   QStringList slist = line.split("=");
                   QString s = slist[1];
                   QStringList list;
                   list<<"1" <<"1.25"<<"1.5" <<"1.75"<<"2" ;
                   switch(list.indexOf(s)){
                           case 0:
                                   ui->ScalingcomboBox->setCurrentIndex(0);
                                   break;
                           case 1:
                                   ui->ScalingcomboBox->setCurrentIndex(1);
                                   break;
                           case 2:
                                   ui->ScalingcomboBox->setCurrentIndex(2);
                                   break;
                           case 3:
                                   ui->ScalingcomboBox->setCurrentIndex(3);
                                   break;
                           case 4:
                                   ui->ScalingcomboBox->setCurrentIndex(4);
                                   break;
                           default:
                                   break;
                   }
                   break;
              }
          }
          file.close();
      }
 }

 void Font::on_scaling_changed()
 {
     Common::ChattrDir chattr;
     QString s = ui->ScalingcomboBox->currentText();
     qInfo()<<"ceshi:"<<s;
     KAuth::Action action("org.nde.auth.helper.setscaling");
     if(!action.isValid())
     {
         qInfo()<<"org.nde.auth.helper.setscaling is valid";
         exit(1);
     }
     action.setHelperId(QStringLiteral("org.nde.auth.helper"));
     action.addArgument(QStringLiteral("FONT_SCALE_LEVEL"),s);
     KAuth::ExecuteJob *job = action.execute();
     job->exec();
 }

