#include "keyboardlayoutlist.h"
#include "ui_keyboardlayoutlist.h"
#include "keyboardlayoutitem.h"
#include <QTextCodec>
#include "keyboardinstance.h"

static bool In(int min, int max, int v)
{
    return (v >= min && v <= max);
}

static char convert(int n)
{
    /*
         * GB2312码范围 (B0A1-F7FE)
         * HiByte (B0-F7) LoByte (A1-FE)
         *
         * GBK编码范围 (8140-FEFE)
         * 包含三大部分
         * 1.汉字区
         *      a. GB2312汉字区。即GBK/2
         *      b. GB13000.1扩充汉字区。包括GBK/3(CJK汉字)和GBK/4(CJK汉字和增补汉字)
         * 2.图形符号区
         *      a. GB2312非汉字区。即GBK/1 A1A1-A9FE。还有10个小写罗马数字和GB12345增补符号
         *      b. GB13000.1扩充非汉字区。即GBK/5 A840-A9A0非汉字符号、结构符...
         * 3.用户自定义区
         *
         */
    if (In(0xB0A1,0xB0C4,n)) return 'A';
    if (In(0XB0C5,0XB2C0,n)) return 'B';
    if (In(0xB2C1,0xB4ED,n)) return 'C';
    if (In(0xB4EE,0xB6E9,n)) return 'D';
    if (In(0xB6EA,0xB7A1,n)) return 'E';
    if (In(0xB7A2,0xB8C0,n)) return 'F';
    if (In(0xB8C1,0xB9FD,n)) return 'G';
    if (In(0xB9FE,0xBBF6,n)) return 'H';
    if (In(0xBBF7,0xBFA5,n)) return 'J';
    if (In(0xBFA6,0xC0AB,n)) return 'K';
    if (In(0xC0AC,0xC2E7,n)) return 'L';
    if (In(0xC2E8,0xC4C2,n)) return 'M';
    if (In(0xC4C3,0xC5B5,n)) return 'N';
    if (In(0xC5B6,0xC5BD,n)) return 'O';
    if (In(0xC5BE,0xC6D9,n)) return 'P';
    if (In(0xC6DA,0xC8BA,n)) return 'Q';
    if (In(0xC8BB,0xC8F5,n)) return 'R';
    if (In(0xC8F6,0xCBF9,n)) return 'S';
    if (In(0xCBFA,0xCDD9,n)) return 'T';
    if (In(0xCDDA,0xCEF3,n)) return 'W';
    if (In(0xCEF4,0xD1B8,n)) return 'X';
    if (In(0xD1B9,0xD4D0,n)) return 'Y';
    if (In(0xD4D1,0xD7F9,n)) return 'Z';
    if (In(0x00, 0x7f, n)) return n;
    return '\0';
}

QString ConvertToQuanpin(const QString zhStr)
{
    QString quanpin;
    for(int i = 0; i < zhStr.length(); i++)
    {
        QString str = zhStr.at(i);
        QTextCodec *pCodec = QTextCodec::codecForName("gb2312");
        if(!pCodec) return QChar(' ');
        QByteArray arr = pCodec->fromUnicode(str);
        wchar_t wchr = 0;

        if(arr.size() == 1)
        {
            wchr = arr.at(0) & 0xff;
        }
        else if (arr.size() == 2)
        {
            wchr = (arr.at(0) & 0xff) << 8;
            wchr |= (arr.at(1) & 0xff);
        }
        else
        {

        }

        char c = convert(wchr);
        if(c != 0)
        {
            quanpin.append(c);
        }
    }
    return quanpin;
}

bool checkSearchFit(const QString &source, const QString &filter)
{
    QString tempFilter = filter;
    QString nospaceFilter = tempFilter.remove(QRegExp("\\s"));
    if(nospaceFilter.isEmpty()){
        return true;
    }

    QString source_string = source;
    QString pinyin_string = ConvertToQuanpin(source_string);
    if(source_string.contains(nospaceFilter, Qt::CaseInsensitive))
    {
        return true;
    }

    if(pinyin_string.contains(nospaceFilter, Qt::CaseInsensitive))
    {
        return true;
    }

    return false;
}



KeyboardLayoutList::KeyboardLayoutList(QWidget *parent) :
    Page(parent),
    ui(new Ui::KeyboardLayoutList)
{
    ui->setupUi(this);

    QFile f(":/qss/keyboardlayoutlist.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }

    connect(ui->lineEdit_search, &QLineEdit::textChanged, this, &KeyboardLayoutList::onSearchTextChanged);
    connect(KeyboardInstance::getIns(), &KeyboardInstance::signalSelKeyboardLayoutsChanged,
            this, &KeyboardLayoutList::onSelKeyboardLayoutsChanged);
    loadKeyboardLayout();
}

KeyboardLayoutList::~KeyboardLayoutList()
{
    delete ui;
}

void KeyboardLayoutList::refreshPage()
{
    //
    ui->lineEdit_search->setText("");
    this->onSelKeyboardLayoutsChanged();
}

void KeyboardLayoutList::loadKeyboardLayout()
{
    while(ui->layout_list->count())
    {
        QLayoutItem * item = ui->layout_list->takeAt(0);
        if(item->widget()) item->widget()->deleteLater();
        delete item;
    }

    QMap<QString, KeyboardLayoutInfo > allKeyboardLayouts = KeyboardInstance::getIns()->getAllKeyboardLayout();

    QMap<QString, KeyboardLayoutInfo >::const_iterator it;
    for(it = allKeyboardLayouts.constBegin(); it != allKeyboardLayouts.constEnd(); ++it)
    {
        const KeyboardLayoutInfo& info = *it;

        KeyboardLayoutItem* listItem = new KeyboardLayoutItem (info.description, "", KEYBOART_TYPE::Select, this);
        listItem->setKeyboardLayoutData(it.key());
        listItem->setKeyboardVariantData("");
        listItem->setChecked(KeyboardInstance::getIns()->isKeyboardLayoutSelected(it.key(), ""));
        connect(listItem, &KeyboardLayoutItem::selectedKeyboardLayoutItem,this, &KeyboardLayoutList::onSelectedKeyboardLayoutItem);
        ui->layout_list->addWidget(listItem);

        QList<LayoutVariantInfo>::const_iterator var;
        for(var = info.variants.constBegin(); var != info.variants.constEnd(); ++var)
        {
            KeyboardLayoutItem* variantItem = new KeyboardLayoutItem (info.description, var->description, KEYBOART_TYPE::Select, this);
            variantItem->setChecked(KeyboardInstance::getIns()->isKeyboardLayoutSelected(it.key(), var->name));
            variantItem->setKeyboardLayoutData(it.key());
            variantItem->setKeyboardVariantData(var->name);
            connect(variantItem, &KeyboardLayoutItem::selectedKeyboardLayoutItem,this, &KeyboardLayoutList::onSelectedKeyboardLayoutItem);
            ui->layout_list->addWidget(variantItem);
        }
    }
}

void KeyboardLayoutList::onSearchTextChanged(const QString &filter)
{
    for(int i = 0; i < ui->layout_list->count(); i++)
    {
        KeyboardLayoutItem* layoutItem = qobject_cast<KeyboardLayoutItem*>(ui->layout_list->itemAt(i)->widget());
        QString layoutData = layoutItem->layouts();

        if(checkSearchFit(layoutData, filter))
        {
            layoutItem->show();
        }
        else
        {
            layoutItem->hide();
        }
    }
}

void KeyboardLayoutList::onSelectedKeyboardLayoutItem()
{
    KeyboardLayoutItem* keyItem = qobject_cast<KeyboardLayoutItem*>(sender());
    if(keyItem)
    {
        if(keyItem->type() != KEYBOART_TYPE::Select)
        {
            return;
        }

        QString layout = keyItem->getKeyboardLayoutData();
        QString variant = keyItem->getKeyboardVariantData();
        if(keyItem->checked())
        {
            KeyboardInstance::getIns()->addKeyboardLayout(layout, variant);
        }
        else
        {
            KeyboardInstance::getIns()->removeKeyboardLayout(layout, variant);
        }
    }
}

void KeyboardLayoutList::onSelKeyboardLayoutsChanged()
{
    for(int i = 0; i < ui->layout_list->count(); i++)
    {
        KeyboardLayoutItem* layoutItem = qobject_cast<KeyboardLayoutItem*>(ui->layout_list->itemAt(i)->widget());
        QString layout = layoutItem->getKeyboardLayoutData();
        QString variant = layoutItem->getKeyboardVariantData();
        layoutItem->setChecked(KeyboardInstance::getIns()->isKeyboardLayoutSelected(layout, variant));
    }
}
