#include "power.h"
#include "ui_power.h"
#include "batteryhelper.h"
#include <QProcess>
#include <Nde/Notification>
#include <Nde/AutostartEntry>
#include <Nde/Globals>
#include <Nde/Power>
#include <QFile>

Power::Power(QWidget *parent) :
    Page(parent),
    ui(new Ui::Power),
    mBattery(nullptr),
    mSettings(),
    mEnableBatteryToggle(nullptr),
    mEnableLidToggle(nullptr),
    mUPowerInterface(nullptr)
{
    ui->setupUi(this);
    QFile f(":/qss/power.qss");
    if(f.open(QIODevice::ReadOnly))
    {
        QString style = f.readAll().replace("\n", "");
        this->setStyleSheet(style);
    }

    const QList<Solid::Device> devices = Solid::Device::listFromType(Solid::DeviceInterface::Battery, QString());
    for (Solid::Device device : devices)
    {
        Solid::Battery *battery = device.as<Solid::Battery>();
        if (battery->type() != Solid::Battery::PrimaryBattery)
        {
            continue;
        }
        mBatteries << battery;
        connect(battery, &Solid::Battery::energyChanged, this, &Power::updatePowerType);
        connect(battery, &Solid::Battery::chargeStateChanged, this, &Power::updatePowerType);
    }
    BatteryHelper::batterys = mBatteries;
    mBattery = mBatteries[0];

    initUI();
}

Power::~Power()
{
    delete ui;
    delete mEnableBatteryToggle;
    delete mEnableLidToggle;
    mEnableBatteryToggle = nullptr;
    mEnableLidToggle = nullptr;
    delete mUPowerInterface;
    mUPowerInterface = nullptr;
}

void Power::refreshPage()
{
    //
}

void Power::initUI()
{
    ui->frame_1->hide();  // 隐藏电池监控开关功能
    updatePowerType();

    // 是否开启电池设置
    QHBoxLayout * enableBatterLayout = new QHBoxLayout();
    enableBatterLayout->setContentsMargins(0,0,0,0);
    enableBatterLayout->setSpacing(0);
    ui->enableBatterWidget->setLayout(enableBatterLayout);
    mEnableBatteryToggle = new NdeWidget::QMaterialToggle(ui->enableBatterWidget);
    mEnableBatteryToggle->setChecked(mSettings.isBatteryWatcherEnabled());
    enableBatterLayout->addWidget(mEnableBatteryToggle);
    connect(mEnableBatteryToggle, &NdeWidget::QMaterialToggle::toggled, this, &Power::onEnableBatteryChanged);

    if(mSettings.isBatteryWatcherEnabled())
    {
        ui->frame_2->show();
        ui->frame_3->show();
        ui->frame_4->show();
    }
    else
    {
        ui->frame_2->hide();
        ui->frame_3->hide();
        ui->frame_4->hide();
    }
    fillComboBox(ui->actionComboBox);
    connect(ui->actionComboBox, SIGNAL(activated(int)), this, SLOT(saveSettings()));
    connect(ui->warningSpinBox, SIGNAL(editingFinished()), this, SLOT(saveSettings()));
    connect(ui->levelSpinBox, SIGNAL(editingFinished()), this, SLOT(saveSettings()));

    bool hasBattery = false;

    const auto devices = Solid::Device::listFromType(Solid::DeviceInterface::Battery, QString());
    for (const Solid::Device& device : devices)
        if (device.as<Solid::Battery>()->type() == Solid::Battery::PrimaryBattery)
            hasBattery = true;
    if(!hasBattery) // 无电池则不显示
    {
        ui->label_3->hide();
        ui->frame->hide();
        ui->frame_1->hide();
        ui->frame_2->hide();
        ui->frame_3->hide();
        ui->frame_4->hide();
    }

    // 是否开启屏盖设置
    QHBoxLayout * enableLidLayout = new QHBoxLayout();
    enableLidLayout->setContentsMargins(0,0,0,0);
    enableLidLayout->setSpacing(0);
    ui->enabelLidWidget->setLayout(enableLidLayout);
    mEnableLidToggle = new NdeWidget::QMaterialToggle(ui->enabelLidWidget);
    mEnableLidToggle->setChecked(mSettings.isLidWatcherEnabled());
    enableLidLayout->addWidget(mEnableLidToggle);
    connect(mEnableLidToggle, &NdeWidget::QMaterialToggle::toggled, this, &Power::onEnableLidChanged);

    if(mSettings.isLidWatcherEnabled())
    {
        ui->frame_7->show();
        ui->frame_8->show();
    }
    else
    {
        ui->frame_7->hide();
        ui->frame_8->hide();
    }
    fillComboBox(ui->onBatteryActionComboBox);
    fillComboBox(ui->onAcActionComboBox);
    connect(ui->onBatteryActionComboBox, SIGNAL(activated(int)), this, SLOT(saveSettings()));
    connect(ui->onAcActionComboBox, SIGNAL(activated(int)), this, SLOT(saveSettings()));
    loadSettings();

    mUPowerInterface = new QDBusInterface(QL1S("org.freedesktop.UPower"),
                                                          QL1S("/org/freedesktop/UPower"),
                                                          QL1S("org.freedesktop.UPower"),
                                                          QDBusConnection::systemBus(), this);
    if(!mUPowerInterface->property("LidIsPresent").toBool()) // 无盖屏则不显示
    {
        ui->label_7->hide();
        ui->frame_6->hide();
        ui->frame_7->hide();
        ui->frame_8->hide();
    }
}

void Power::iconChanged()
{
    int ChargePercent = BatteryHelper::getBatteryChargecent();
    Solid::Battery::ChargeState state = BatteryHelper::getBatteryState();
    ui->iconLabel->setPixmap(icontool.getIcon(ChargePercent,state));
}

void Power::updatePowerType()
{
    int ChargePercent = BatteryHelper::getBatteryChargecent();
    Solid::Battery::ChargeState state = BatteryHelper::getBatteryState();
    ui->iconLabel->setPixmap(icontool.getIcon(ChargePercent,state));
    ui->typeLabel->setText(QString("(") + BatteryHelper::stateToString(state) + QString(")"));
    ui->electricityProgressBar->setValue(ChargePercent);
    ui->numBerLabel->setText(QString("%1%").arg(ChargePercent));
}

void Power::onEnableBatteryChanged(bool checked)
{
    if(checked)
    {
        ui->frame_2->show();
        ui->frame_3->show();
        ui->frame_4->show();
    }
    else
    {
        ui->frame_2->hide();
        ui->frame_3->hide();
        ui->frame_4->hide();
    }
    saveSettings();
}
void Power::onEnableLidChanged(bool checked)
{
    if(checked)
    {
        ui->frame_7->show();
        ui->frame_8->show();
    }
    else
    {
        ui->frame_7->hide();
        ui->frame_8->hide();
    }
    saveSettings();
}


void Power::fillComboBox(QComboBox* comboBox)
{
    comboBox->clear();
    comboBox->addItem(tr("Nothing"), -1);
    comboBox->addItem(tr("Lock screen"), -2); // FIXME
    comboBox->addItem(tr("Suspend"), Nde::Power::PowerSuspend);
//    comboBox->addItem(tr("Hibernate"), Nde::Power::PowerHibernate);
    comboBox->addItem(tr("Shutdown"), Nde::Power::PowerShutdown);
    comboBox->addItem(tr("Turn Off monitor(s)"), Nde::Power::PowerMonitorOff);
}

void Power::setComboBoxToValue(QComboBox* comboBox, int value)
{
    for (int index = 0; index < comboBox->count(); index++)
    {
        if (value == comboBox->itemData(index).toInt())
        {
            comboBox->setCurrentIndex(index);
            return;
        }
    }

    comboBox->setCurrentIndex(0);
}

int Power::currentValue(QComboBox *comboBox)
{
    return comboBox->itemData(comboBox->currentIndex()).toInt();
}

void Power::saveSettings()
{
    mSettings.setBatteryWatcherEnabled(mEnableBatteryToggle->isChecked());
    mSettings.setPowerLowAction(currentValue(ui->actionComboBox));
    mSettings.setPowerLowWarningTime(ui->warningSpinBox->value());
    mSettings.setPowerLowLevel(ui->levelSpinBox->value());

    mSettings.setLidWatcherEnabled(mEnableLidToggle->isChecked());
    mSettings.setLidClosedAction(currentValue(ui->onBatteryActionComboBox));
    mSettings.setLidClosedAcAction(currentValue(ui->onAcActionComboBox));
}

void Power::loadSettings()
{
    setComboBoxToValue(ui->actionComboBox, mSettings.getPowerLowAction());
    ui->warningSpinBox->setValue(mSettings.getPowerLowWarningTime());
    ui->levelSpinBox->setValue(mSettings.getPowerLowLevel());

    setComboBoxToValue(ui->onBatteryActionComboBox, mSettings.getLidClosedAction());
    setComboBoxToValue(ui->onAcActionComboBox, mSettings.getLidClosedAcAction());
}
