#include "applicationframe.h"
#include <XdgIcon>
#include <QtDBus/QDBusInterface>

//CommonAppData排序函数
bool compareFixedAppData(const FixedAppData & f1, const FixedAppData & f2)
{
    return f1._fixedTime > f2._fixedTime;
}

QString getDFID(const XdgDesktopFile d)
{
    QStringList list = d.fileName().split("/");
    if(list.isEmpty()) return QString();
    return list.takeLast();
}

ApplicationFrame::ApplicationFrame(const XdgDesktopFile &desktopFile, QFrame *parent):
    QFrame(parent),
    hLayout_(nullptr),
    iconLabel_(nullptr),
    textLabel_(nullptr),
    spacer_(nullptr),
    enableLayout_(nullptr),
    toggleEnabel_(nullptr),
    enableWidget_(nullptr),
    panelSetting_(nullptr)
{
    isCommon_ = true;
    commonSetting_ = new Nde::Settings(QLatin1String("mainmenu"), this);
    parseFixedSetting();
    initUI();
    load(desktopFile);
}

ApplicationFrame::ApplicationFrame(const SystemIconData &systemApp, QFrame *parent):
    QFrame(parent),
    hLayout_(nullptr),
    iconLabel_(nullptr),
    textLabel_(nullptr),
    spacer_(nullptr),
    enableLayout_(nullptr),
    toggleEnabel_(nullptr),
    enableWidget_(nullptr),
    commonSetting_(nullptr),
    panelSetting_(nullptr)
{
    isShowSystemIcon_ = true;
    systemIconSetting_ = new Nde::Settings(QLatin1String("nde-systemicon"), this);
    initUI();
    systemAppLoad(systemApp);
}

ApplicationFrame::ApplicationFrame(const QString &plugin, const QString &text, QFrame *parent):
    QFrame(parent),
    hLayout_(nullptr),
    iconLabel_(nullptr),
    textLabel_(nullptr),
    spacer_(nullptr),
    enableLayout_(nullptr),
    toggleEnabel_(nullptr),
    enableWidget_(nullptr),
    commonSetting_(nullptr),
    systemIconSetting_(nullptr)
{
    panelSetting_ = new Nde::Settings(QLatin1String("panel"), this);
    isPlugin_ = true;
    initUI();
    pluginLoad(plugin, text);
}

ApplicationFrame::~ApplicationFrame()
{

}

void ApplicationFrame::initUI()
{
    iconLabel_ = new QLabel(this);
    iconLabel_->setObjectName("label_icon");
    iconLabel_->setFixedSize(32, 32);

    textLabel_ = new QLabel(this);
    textLabel_->setObjectName("label_text");
    textLabel_->setFixedHeight(32);
    textLabel_->setTextFormat(Qt::PlainText);
    textLabel_->setWordWrap(false);
    textLabel_->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);

    spacer_ = new QSpacerItem(20, 20, QSizePolicy::Expanding);

    enableLayout_ = new QHBoxLayout();
    enableLayout_->setContentsMargins(0,0,0,0);
    enableLayout_->setSpacing(0);
    enableWidget_ = new QWidget();
    enableWidget_->setLayout(enableLayout_);
    enableWidget_->setMaximumSize(QSize(46, 20));
    toggleEnabel_ = new NdeWidget::QMaterialToggle(enableWidget_);
    toggleEnabel_->setChecked(false);
    enableLayout_->addWidget(toggleEnabel_);
    connect(toggleEnabel_, &NdeWidget::QMaterialToggle::toggled, this, &ApplicationFrame::onToggleEnableChanged);

    hLayout_ = new QHBoxLayout();
    hLayout_->setContentsMargins(14, 9, 14, 9); //设置框内的间距
    hLayout_->addWidget(iconLabel_);
    hLayout_->addWidget(textLabel_);
    hLayout_->addSpacerItem(spacer_);
    hLayout_->addWidget(enableWidget_);

    setLayout(hLayout_);
    setStyleSheet("QFrame{border:none; border-radius:4px; background: #f0f1f2}");
    setMinimumHeight(50);
    setMaximumHeight(50);
    setMinimumWidth(300);
}

void ApplicationFrame::load(const XdgDesktopFile &desktopFile)
{
    desktopFile_ = desktopFile;
    if (desktopFile_.isValid())
    {
        // & is reserved for mnemonics
        QString text = desktopFile_.name().replace(QLatin1Char('&'), QLatin1String("&&"));
        textLabel_->setText(text);

        QIcon icon = desktopFile_.icon().isNull() ? XdgIcon::fromTheme(QLatin1String("application-x-executable")) : desktopFile_.icon();
        // Some themes may lack the "application-x-executable" icon; checking null prevents
        // infinite recursion (setIcon->dataChanged->updateIcon->setIcon
         if (!icon.isNull()) {
             QPixmap p = icon.pixmap(iconLabel_->width());
             iconLabel_->setPixmap(p);
         }

         setToolTip(desktopFile_.comment());
         FixedAppData tmpF;
         tmpF._desktopFile = desktopFile_;
         for(FixedAppData f : fixedAppDatas_)
         {
             if(tmpF == f)
             {
                toggleEnabel_->setChecked(true);
                isCommon_ = true;
                 break;
             }
         }
    }
    else
    {
        textLabel_->setText(QString());
        setToolTip(QString());
        iconLabel_->setPixmap(QPixmap());
    }
}

void ApplicationFrame::systemAppLoad(SystemIconData appName)
{
    systemApp_ = appName;
    textLabel_->setText(appName.name);
    QIcon icon;
    if (QIcon::hasThemeIcon(appName.icon))
        icon = QIcon::fromTheme(appName.icon);
    else {
        icon = QIcon();
    }

    if (!icon.isNull()) {
        QPixmap p = icon.pixmap(iconLabel_->width());
        iconLabel_->setPixmap(p);
    }
    systemIconSetting_->beginGroup(appName.command);
    bool display = systemIconSetting_->value("Dispaly", true).toBool();
    toggleEnabel_->setChecked(display);
    systemIconSetting_->endGroup();
    if(systemApp_.command == "nde-powermanagement")
    {
        Nde::Settings * powersetting = new Nde::Settings(QLatin1String("nde-powermanagement"), this);
        bool enableBatteryWatcher = powersetting->value("enableBatteryWatcher", false).toBool();
        if(!enableBatteryWatcher){
            toggleEnabel_->setEnabled(enableBatteryWatcher);
            textLabel_->setStyleSheet(QLatin1String("QLabel {color: #a8abb1;}"));
        }
    }
}

void ApplicationFrame::pluginLoad(QString plugin, QString text)
{
    plugin_ = plugin;
    textLabel_->setText(text);
    iconLabel_->hide();
    pluginChecked_ = panelSetting_->value(plugin + QString("/visible"), true).toBool();
    toggleEnabel_->setChecked(pluginChecked_);
}

void ApplicationFrame::onToggleEnableChanged(bool checked)
{
    if(isShowSystemIcon_)
    {
        systemIconSetting_->beginGroup(systemApp_.command);
        systemIconSetting_->setValue("Command", systemApp_.name);
        systemIconSetting_->setValue("Icon", systemApp_.icon);
        systemIconSetting_->setValue("Dispaly", checked);
        systemIconSetting_->endGroup();
        systemIconSetting_->sync();
        QDBusInterface* interface = new QDBusInterface(QString::fromLatin1("com.nde.panel.statusnotifiertray"),
                                                     QString::fromLatin1("/com/nde/panel/statusnotifiertray"),
                                                     QString::fromLatin1("com.nde.panel.statusnotifiertray"),
                                                     QDBusConnection::sessionBus());
        interface->call("onHideOrShowSystemIconCheange", systemApp_.command, checked);
        delete interface;
    }
    else if(isPlugin_)
    {
        pluginChecked_ = checked;
        QDBusInterface* interface = new QDBusInterface(QString::fromLatin1("com.nde.panel.ndepanel"),
                                                     QString::fromLatin1("/com/nde/panel/ndepanel"),
                                                     QString::fromLatin1("com.nde.panel.ndepanel"),
                                                     QDBusConnection::sessionBus());
        interface->call("onpluginVisible", plugin_);
        delete interface;
    }
    else {
        isCommon_ = checked;
        QDBusInterface* interface = new QDBusInterface(QString::fromLatin1("org.nde.AppManager"),
                                                     QString::fromLatin1("/com/nde/panel/mainmenu"),
                                                     QString::fromLatin1("com.nde.panel.mainmenu"),
                                                     QDBusConnection::sessionBus());
        interface->call("systemCommonAppFixed", desktopFile_.fileName());
        delete interface;
    }
}

void ApplicationFrame::saveFixedSetting()
{
    commonSetting_->beginGroup("fixed_apps");

    QStringList keys = commonSetting_->childKeys();
    for(QString key : keys)
    {
        commonSetting_->remove(key);
    }

    for(FixedAppData f: fixedAppDatas_)
    {
        QString key = getDFID(f._desktopFile);
        commonSetting_->setValue(key, f._fixedTime.toString("yyyy/MM/dd HH:mm:ss"));
    }
    commonSetting_->endGroup();

    commonSetting_->sync();
}

void ApplicationFrame::parseFixedSetting()
{
    fixedAppDatas_.clear();
    //读取配置
    commonSetting_->beginGroup("fixed_apps");
    QStringList keys = commonSetting_->childKeys();
    for(QString key : keys)
    {
        FixedAppData f;
        QString v = commonSetting_->value(key).toString();
        f._desktopFile.load(key);
        f._fixedTime = QDateTime::fromString(v,"yyyy/MM/dd HH:mm:ss");

        //判断该desktop所在的文件是否存在
        if(XdgDesktopFile::id(f._desktopFile.fileName()).isEmpty())
        {
            commonSetting_->remove(key);
            continue;
        }

        fixedAppDatas_.append(f);
    }
    commonSetting_->endGroup();

    //按时间从最近到到最早排序
    qSort(fixedAppDatas_.begin(), fixedAppDatas_.end(), compareFixedAppData);
}

void ApplicationFrame::setCommonChecked()
{
    toggleEnabel_->setChecked(!isCommon_);
    isCommon_ = !isCommon_;
}

void ApplicationFrame::setPluginChecked()
{
    toggleEnabel_->setChecked(!pluginChecked_);
    pluginChecked_ = !pluginChecked_;
}
